/*
 * segatex_editor SELinux tool.
 *
 * Copyright (C) 2007 Shintaro Fujiwara 
 * shintaro.fujiwara@gmai.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * */

#include <qaction.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qworkspace.h>
//newly added
#include <qsplitter.h>
#include <qpushbutton.h>
#include <qtextedit.h>

#include "editor.h"
#include "parselines.h"
#include "parselines_admin.h"
#include "parselines_apps.h"
#include "parselines_kernel.h"
#include "parselines_services.h"
#include "parselines_system.h"
#include "parselines_misc.h"
#include "parselines_selpaw.h"
#include "selpaw.h"
#include "selpaw_adjust.h"
#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent, const char *name)
    : QMainWindow(parent, name)
{
    //newly added
    //declare horizontalSplitter and set it central
    horizontalSplitter = new QSplitter(Horizontal, this);
    setCentralWidget(horizontalSplitter);
    //declare workspace as a child of horizontalSplitter
    workspace = new QWorkspace(horizontalSplitter);
    //declare vertivalSplitter as a child of horizontalSpliter and prepare
    verticalSplitter = new QSplitter(Vertical, horizontalSplitter);
    verticalSplitter2 = new QSplitter(Vertical, horizontalSplitter);
    //declare pushButtons and Parse objects as a variable of verticalSplitter
    //pushButton22 = new QPushButton("refine_selpaw", verticalSplitter2);
    
    parse1 = new ParseLinesAdmin();
    parse2 = new ParseLinesApps();
    parse3 = new ParseLinesKernel();
    parse4 = new ParseLinesServices();
    parse5 = new ParseLinesSystem();
    parse6 = new ParseLinesMisc();
    parse9 = new ParseLinesAdmin();
    parse10 = new ParseLinesApps();
    parse11 = new ParseLinesKernel();
    parse12 = new ParseLinesServices();
    parse13 = new ParseLinesSystem();    
    parse14 = new ParseLinesSelpaw();
    selpaw1 = new Selpaw();
    selpawadjust1 = new SelpawAdjust();
 
    //connect SIGNAL to Parse objects    
    //connect(pushButton19, SIGNAL(clicked()),selpaw1, SLOT(make_refine_selpaw()));
    
    connect(workspace, SIGNAL(windowActivated(QWidget *)),
            this, SLOT(updateMenus()));
    connect(workspace, SIGNAL(windowActivated(QWidget *)),
            this, SLOT(updateModIndicator()));

    createActions();
    createMenus();
    createToolBars();
    createStatusBar();

    setCaption(tr("segatex_editor-3.2"));
    setIcon(QPixmap::fromMimeSource("icon.png"));
}

void MainWindow::createActions()
{
    newAct = new QAction(tr("&New"), tr("Ctrl+N"), this);
    newAct->setIconSet(QPixmap::fromMimeSource("new.png"));
    newAct->setStatusTip(tr("Create a new file"));
    connect(newAct, SIGNAL(activated()), this, SLOT(newFile()));

    openAct = new QAction(tr("&Open..."), tr("Ctrl+O"), this);
    openAct->setIconSet(QPixmap::fromMimeSource("open.png"));
    openAct->setStatusTip(tr("Open an existing file"));
    connect(openAct, SIGNAL(activated()), this, SLOT(open()));

    saveAct = new QAction(tr("&Save"), tr("Ctrl+S"), this);
    saveAct->setIconSet(QPixmap::fromMimeSource("save.png"));
    saveAct->setStatusTip(tr("Save the file to disk"));
    connect(saveAct, SIGNAL(activated()), this, SLOT(save()));

    saveAsAct = new QAction(tr("Save &As..."), 0, this);
    saveAsAct->setStatusTip(tr("Save the file under a new name"));
    connect(saveAsAct, SIGNAL(activated()), this, SLOT(saveAs()));

    exitAct = new QAction(tr("E&xit"), tr("Ctrl+Q"), this);
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(activated()), this, SLOT(close()));

    cutAct = new QAction(tr("Cu&t"), tr("Ctrl+X"), this);
    cutAct->setIconSet(QPixmap::fromMimeSource("cut.png"));
    cutAct->setStatusTip(tr("Cut the current selection to the "
                            "clipboard"));
    connect(cutAct, SIGNAL(activated()), this, SLOT(cut()));

    copyAct = new QAction(tr("&Copy"), tr("Ctrl+C"), this);
    copyAct->setIconSet(QPixmap::fromMimeSource("copy.png"));
    copyAct->setStatusTip(tr("Copy the current selection to the "
                             "clipboard"));
    connect(copyAct, SIGNAL(activated()), this, SLOT(copy()));

    pasteAct = new QAction(tr("&Paste"), tr("Ctrl+V"), this);
    pasteAct->setIconSet(QPixmap::fromMimeSource("paste.png"));
    pasteAct->setStatusTip(tr("Paste the clipboard's contents at "
                              "the cursor position"));
    connect(pasteAct, SIGNAL(activated()), this, SLOT(paste()));

    deleteAct = new QAction(tr("&Delete"), tr("Del"), this);
    deleteAct->setIconSet(QPixmap::fromMimeSource("delete.png"));
    deleteAct->setStatusTip(tr("Delete the current selection"));
    connect(deleteAct, SIGNAL(activated()), this, SLOT(del()));

    closeAct = new QAction(tr("Cl&ose"), tr("Ctrl+F4"), this);
    closeAct->setStatusTip(tr("Close the active window"));
    connect(closeAct, SIGNAL(activated()),
            workspace, SLOT(closeActiveWindow()));

    closeAllAct = new QAction(tr("Close &All"), 0, this);
    closeAllAct->setStatusTip(tr("Close all the windows"));
    connect(closeAllAct, SIGNAL(activated()),
            workspace, SLOT(closeAllWindows()));

    tileAct = new QAction(tr("&Tile"), 0, this);
    tileAct->setStatusTip(tr("Tile the windows"));
    connect(tileAct, SIGNAL(activated()), workspace, SLOT(tile()));

    cascadeAct = new QAction(tr("&Cascade"), 0, this);
    cascadeAct->setStatusTip(tr("Cascade the windows"));
    connect(cascadeAct, SIGNAL(activated()),
            workspace, SLOT(cascade()));

    nextAct = new QAction(tr("Ne&xt"), tr("Ctrl+F6"), this);
    nextAct->setStatusTip(tr("Move the focus to the next window"));
    connect(nextAct, SIGNAL(activated()),
            workspace, SLOT(activateNextWindow()));

    previousAct = new QAction(tr("Pre&vious"),
                              tr("Ctrl+Shift+F6"), this);
    previousAct->setStatusTip(tr("Move the focus to the previous "
                                 "window"));
    connect(previousAct, SIGNAL(activated()),
            workspace, SLOT(activatePreviousWindow()));

    aboutAct = new QAction(tr("&About"), 0, this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(activated()), this, SLOT(about()));


    aboutQtAct = new QAction(tr("About &Qt"), 0, this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(activated()), qApp, SLOT(aboutQt()));

    //newly added-1:create action
    parse_adminAct = new QAction(tr("&Parse Admin"), 0, this);
    parse_adminAct->setStatusTip(tr("Parse .if files in admin layer"));
    connect(parse_adminAct, SIGNAL(activated()), parse1, SLOT(parse_admin()));
    connect(parse_adminAct, SIGNAL(activated()), this, SLOT(open_interfaces_admin_txt()));

    parse_appsAct = new QAction(tr("&Parse Apps"), 0, this);
    parse_appsAct->setStatusTip(tr("Parse .if files in apps layer"));
    connect(parse_appsAct, SIGNAL(activated()), parse2, SLOT(parse_apps()));
    connect(parse_appsAct, SIGNAL(activated()), this, SLOT(open_interfaces_apps_txt()));

    parse_kernelAct = new QAction(tr("&Parse Kernel"), 0, this);
    parse_kernelAct->setStatusTip(tr("Parse .if files in kernel layer"));
    connect(parse_kernelAct, SIGNAL(activated()), parse3, SLOT(parse_kernel()));
    connect(parse_kernelAct, SIGNAL(activated()), this, SLOT(open_interfaces_kernel_txt()));

    parse_servicesAct = new QAction(tr("&Parse Services"), 0, this);
    parse_servicesAct->setStatusTip(tr("Parse .if files in services layer"));
    connect(parse_servicesAct, SIGNAL(activated()), parse4, SLOT(parse_services()));
    connect(parse_servicesAct, SIGNAL(activated()), this, SLOT(open_interfaces_services_txt()));

    parse_systemAct = new QAction(tr("&Parse System"), 0, this);
    parse_systemAct->setStatusTip(tr("Parse .if files in system layer"));
    connect(parse_systemAct, SIGNAL(activated()), parse5, SLOT(parse_system()));
    connect(parse_systemAct, SIGNAL(activated()), this, SLOT(open_interfaces_system_txt()));

    obj_perm_setsAct = new QAction(tr("&Obj perm sets"), 0, this);
    obj_perm_setsAct->setStatusTip(tr("Parse obj_perm_sets file"));
    connect(obj_perm_setsAct, SIGNAL(activated()), parse6, SLOT(obj_perm_sets()));
    connect(obj_perm_setsAct, SIGNAL(activated()), this, SLOT(open_obj_perm_sets_prepare_txt()));

    ipc_patternsAct = new QAction(tr("&Ipc patterns"), 0, this);
    ipc_patternsAct->setStatusTip(tr("Parse ipc patterns file"));
    connect(ipc_patternsAct, SIGNAL(activated()), parse6, SLOT(ipc_patterns()));
    connect(ipc_patternsAct, SIGNAL(activated()), this, SLOT(open_ipc_patterns_prepare_txt()));

    misc_patternsAct = new QAction(tr("&Misc patterns"), 0, this);
    misc_patternsAct->setStatusTip(tr("Parse misc patterns file"));
    connect(misc_patternsAct, SIGNAL(activated()), parse6, SLOT(misc_patterns()));
    connect(misc_patternsAct, SIGNAL(activated()), this, SLOT(open_misc_patterns_prepare_txt()));

    break_adminAct = new QAction(tr("&Break Admin"), 0, this);
    break_adminAct->setStatusTip(tr("Break .if files in admin layer"));
    connect(break_adminAct, SIGNAL(activated()), parse9, SLOT(break_admin()));
    connect(break_adminAct, SIGNAL(activated()), this, SLOT(open_interfaces_admin_broken_up_txt()));

    break_appsAct = new QAction(tr("&Break Apps"), 0, this);
    break_appsAct->setStatusTip(tr("Break .if files in apps layer"));
    connect(break_appsAct, SIGNAL(activated()), parse10, SLOT(break_apps()));
    connect(break_appsAct, SIGNAL(activated()), this, SLOT(open_interfaces_apps_broken_up_txt()));

    break_kernelAct = new QAction(tr("&Break Kernel"), 0, this);
    break_kernelAct->setStatusTip(tr("Break .if files in kernel layer"));
    connect(break_kernelAct, SIGNAL(activated()), parse11, SLOT(break_kernel()));
    connect(break_kernelAct, SIGNAL(activated()), this, SLOT(open_interfaces_kernel_broken_up_txt()));

    break_servicesAct = new QAction(tr("&Break Services"), 0, this);
    break_servicesAct->setStatusTip(tr("Break .if files in services layer"));
    connect(break_servicesAct, SIGNAL(activated()), parse12, SLOT(break_services()));
    connect(break_servicesAct, SIGNAL(activated()), this, SLOT(open_interfaces_services_broken_up_txt()));

    break_systemAct = new QAction(tr("&Break System"), 0, this);
    break_systemAct->setStatusTip(tr("Break .if files in system layer"));
    connect(break_systemAct, SIGNAL(activated()), parse13, SLOT(break_system()));
    connect(break_systemAct, SIGNAL(activated()), this, SLOT(open_interfaces_system_broken_up_txt()));

    selpaw_adminAct = new QAction(tr("&Selpaw Admin"), 0, this);
    selpaw_adminAct->setStatusTip(tr("parse .if files into .csv file in admin layer"));
    connect(selpaw_adminAct, SIGNAL(activated()), parse14, SLOT(selpaw_admin()));
    connect(selpaw_adminAct, SIGNAL(activated()), this, SLOT(open_selpaw_admin_txt()));

    selpaw_appsAct = new QAction(tr("&Selpaw Apps"), 0, this);
    selpaw_appsAct->setStatusTip(tr("parse .if files into .csv file in apps layer"));
    connect(selpaw_appsAct, SIGNAL(activated()), parse14, SLOT(selpaw_apps()));
    connect(selpaw_appsAct, SIGNAL(activated()), this, SLOT(open_selpaw_apps_txt()));

    selpaw_kernelAct = new QAction(tr("&Selpaw Kernel"), 0, this);
    selpaw_kernelAct->setStatusTip(tr("parse .if files into .csv file in kernel layer"));
    connect(selpaw_kernelAct, SIGNAL(activated()), parse14, SLOT(selpaw_kernel()));
    connect(selpaw_kernelAct, SIGNAL(activated()), this, SLOT(open_selpaw_kernel_txt()));

    selpaw_servicesAct = new QAction(tr("&Selpaw Services"), 0, this);
    selpaw_servicesAct->setStatusTip(tr("parse .if files into .csv file in services layer"));
    connect(selpaw_servicesAct, SIGNAL(activated()), parse14, SLOT(selpaw_services()));
    connect(selpaw_servicesAct, SIGNAL(activated()), this, SLOT(open_selpaw_services_txt()));

    selpaw_systemAct = new QAction(tr("&Selpaw System"), 0, this);
    selpaw_systemAct->setStatusTip(tr("parse .if files into .csv file in system layer"));
    connect(selpaw_systemAct, SIGNAL(activated()), parse14, SLOT(selpaw_system()));
    connect(selpaw_systemAct, SIGNAL(activated()), this, SLOT(open_selpaw_system_txt()));

    selpaw_interface_rawAct = new QAction(tr("&Selpaw_interface_raw.csv"), 0, this);
    selpaw_interface_rawAct->setStatusTip(tr("Make selpaw_interface_raw.csv"));
    connect(selpaw_interface_rawAct, SIGNAL(activated()), selpaw1, SLOT(make_refine_selpaw()));
    connect(selpaw_interface_rawAct, SIGNAL(activated()), this, SLOT(open_selpaw_interface_raw_csv()));

    selpaw_eliminated_interfacesAct = new QAction(tr("&Selpaw_eliminated_interfaces.csv"), 0, this);
    selpaw_eliminated_interfacesAct->setStatusTip(tr("Make selpaw_eliminated_interfaces.csv"));
    connect(selpaw_eliminated_interfacesAct, SIGNAL(activated()), selpawadjust1, SLOT(adjust_selpaw_ready()));
    connect(selpaw_eliminated_interfacesAct, SIGNAL(activated()), this, SLOT(open_selpaw_eliminated_interfaces_csv()));

    selpaw_interface_readyAct = new QAction(tr("&Selpaw_interface_ready.csv"), 0, this);
    selpaw_interface_readyAct->setStatusTip(tr("Make selpaw_interface_ready.csv"));
    connect(selpaw_interface_readyAct, SIGNAL(activated()), selpawadjust1, SLOT(adjust_selpaw()));
    connect(selpaw_interface_readyAct, SIGNAL(activated()), this, SLOT(open_selpaw_interface_ready_csv()));

    selpaw_interface_csvAct = new QAction(tr("&Selpaw_interface.csv"), 0, this);
    selpaw_interface_csvAct->setStatusTip(tr("Make selpaw_interface.csv"));
    connect(selpaw_interface_csvAct, SIGNAL(activated()), selpawadjust1, SLOT(refine_selpaw()));
    connect(selpaw_interface_csvAct, SIGNAL(activated()), this, SLOT(open_selpaw_interface_csv()));

    howtouseAct = new QAction(tr("&How to use"), 0, this);
    howtouseAct->setStatusTip(tr("Show the application's How to use box"));
    connect(howtouseAct, SIGNAL(activated()), this, SLOT(howtouse()));
}

void MainWindow::createMenus()
{
    fileMenu = new QPopupMenu(this);
    newAct->addTo(fileMenu);
    openAct->addTo(fileMenu);
    saveAct->addTo(fileMenu);
    saveAsAct->addTo(fileMenu);
    fileMenu->insertSeparator();
    exitAct->addTo(fileMenu);

    editMenu = new QPopupMenu(this);
    cutAct->addTo(editMenu);
    copyAct->addTo(editMenu);
    pasteAct->addTo(editMenu);
    deleteAct->addTo(editMenu);

    windowsMenu = new QPopupMenu(this);
    createWindowsMenu();

    helpMenu = new QPopupMenu(this);
    //newly added
    howtouseAct->addTo(helpMenu);
    aboutAct->addTo(helpMenu);
    aboutQtAct->addTo(helpMenu);

    //newly added-2:add actino to menu
    parselinesMenu = new QPopupMenu(this);

    parse_adminAct->addTo(parselinesMenu);
    parse_appsAct->addTo(parselinesMenu);
    parse_kernelAct->addTo(parselinesMenu);
    parse_servicesAct->addTo(parselinesMenu);
    parse_systemAct->addTo(parselinesMenu);
    obj_perm_setsAct->addTo(parselinesMenu);
    ipc_patternsAct->addTo(parselinesMenu);
    misc_patternsAct->addTo(parselinesMenu);
    break_adminAct->addTo(parselinesMenu);
    break_appsAct->addTo(parselinesMenu);
    break_kernelAct->addTo(parselinesMenu);
    break_servicesAct->addTo(parselinesMenu);
    break_systemAct->addTo(parselinesMenu);

    selpawMenu = new QPopupMenu(this);

    selpaw_adminAct->addTo(selpawMenu);
    selpaw_appsAct->addTo(selpawMenu);
    selpaw_kernelAct->addTo(selpawMenu);
    selpaw_servicesAct->addTo(selpawMenu);
    selpaw_systemAct->addTo(selpawMenu);

    refineMenu = new QPopupMenu(this);

    selpaw_interface_rawAct->addTo(refineMenu);
    selpaw_eliminated_interfacesAct->addTo(refineMenu);
    selpaw_interface_readyAct->addTo(refineMenu);
    selpaw_interface_csvAct->addTo(refineMenu);

    menuBar()->insertItem(tr("&File"), fileMenu);
    menuBar()->insertItem(tr("&Edit"), editMenu);
    menuBar()->insertItem(tr("&Windows"), windowsMenu);

    //newly added-3:add action to menubar
    menuBar()->insertSeparator();
    menuBar()->insertItem(tr("&ParseLines"), parselinesMenu);
    menuBar()->insertItem(tr("&Selpaw"), selpawMenu);
    menuBar()->insertItem(tr("&Refine"), refineMenu);

    menuBar()->insertSeparator();
    menuBar()->insertItem(tr("&Help"), helpMenu);
}

void MainWindow::createToolBars()
{
    fileToolBar = new QToolBar(tr("File"), this);
    newAct->addTo(fileToolBar);
    openAct->addTo(fileToolBar);
    saveAct->addTo(fileToolBar);

    editToolBar = new QToolBar(tr("Edit"), this);
    cutAct->addTo(editToolBar);
    copyAct->addTo(editToolBar);
    pasteAct->addTo(editToolBar);
}

void MainWindow::createStatusBar()
{
    readyLabel = new QLabel(tr(" Ready"), this);

    modLabel = new QLabel(tr(" MOD "), this);
    modLabel->setAlignment(AlignHCenter);
    modLabel->setMinimumSize(modLabel->sizeHint());
    modLabel->clear();

    statusBar()->addWidget(readyLabel, 1);
    statusBar()->addWidget(modLabel);
}

void MainWindow::newFile()
{
    Editor *editor = createEditor();
    editor->newFile();
    editor->show();
}

Editor *MainWindow::createEditor()
{
    Editor *editor = new Editor(workspace);
    connect(editor, SIGNAL(copyAvailable(bool)),
            this, SLOT(copyAvailable(bool)));
    connect(editor, SIGNAL(modificationChanged(bool)),
            this, SLOT(updateModIndicator()));
    return editor;
}

void MainWindow::open()
{
    Editor *editor = createEditor();
    if (editor->open())
        editor->show();
    else
        editor->close();
}
//4:newly added
void MainWindow::open_interfaces_admin_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_admin.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_apps_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_apps.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_kernel_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_kernel.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_services_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_services.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_system_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_system.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_obj_perm_sets_prepare_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("obj_perm_sets_prepare.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_ipc_patterns_prepare_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("ipc_patterns_prepare.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_misc_patterns_prepare_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("misc_patterns_prepare.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_admin_broken_up_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_admin_broken_up.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_apps_broken_up_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_apps_broken_up.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_kernel_broken_up_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_kernel_broken_up.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_services_broken_up_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_services_broken_up.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_interfaces_system_broken_up_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("interfaces_system_broken_up.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_admin_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_admin.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_apps_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_apps.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_kernel_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_kernel.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_services_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_services.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_system_txt()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_system.txt"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_interface_raw_csv()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_interface_raw.csv"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_eliminated_interfaces_csv()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_eliminated_interfaces.csv"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_interface_ready_csv()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_interface_ready.csv"))
        editor->show();
    else
        editor->close();
}
void MainWindow::open_selpaw_interface_csv()
{
    Editor *editor = createEditor();
    if (editor->openFile("selpaw_interface.csv"))
        editor->show();
    else
        editor->close();
}


void MainWindow::openFile(const QString &fileName)
{
    Editor *editor = createEditor();
    if (editor->openFile(fileName))
        editor->show();
    else
        editor->close();
}

void MainWindow::save()
{
    if (activeEditor()) {
        activeEditor()->save();
        updateModIndicator();
    }
}

Editor *MainWindow::activeEditor()
{
    return (Editor *)workspace->activeWindow();
}

void MainWindow::saveAs()
{
    if (activeEditor()) {
        activeEditor()->saveAs();
        updateModIndicator();
    }
}

void MainWindow::cut()
{
    if (activeEditor())
        activeEditor()->cut();
}

void MainWindow::copy()
{
    if (activeEditor())
        activeEditor()->copy();
}

void MainWindow::paste()
{
    if (activeEditor())
        activeEditor()->paste();
}

void MainWindow::del()
{
    if (activeEditor())
        activeEditor()->del();
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About segatex_editor"),
            tr("<h2>segatex_editor-3.2</h2>"
               "<p>Copyright &copy; 2007 Shintaro Fujiwara."
               "<p>segatex_editor is a SELinux tool"
               "<b>written with Qt</b>."));
}
//newly added
void MainWindow::howtouse()
{
    QMessageBox::about(this, tr("How to use"),
	    tr("<h2>It's more than an editor</h2>"
		"<p>Push Parselines,Selpaw,Refine"
		"<p>one by one orderly."
		"<p>If you could get three selpaw files,"
		"<p>i.e."
		"<p>selpaw_interface.csv,"
		"<p>selpaw_interface_types.csv,"
		"<p>selpaw_interface_summary.csv,"
		"<p>that is a success."
		"<p>Break kernel may take 30 minutes, and"
		"<p>Refine selpaw takes 10 minutes;)</p>"));

}


void MainWindow::updateMenus()
{
    bool hasEditor = (activeEditor() != 0);
    saveAct->setEnabled(hasEditor);
    saveAsAct->setEnabled(hasEditor);
    pasteAct->setEnabled(hasEditor);
    deleteAct->setEnabled(hasEditor);
    copyAvailable(activeEditor()
                  && activeEditor()->hasSelectedText());
    closeAct->setEnabled(hasEditor);
    closeAllAct->setEnabled(hasEditor);
    tileAct->setEnabled(hasEditor);
    cascadeAct->setEnabled(hasEditor);
    nextAct->setEnabled(hasEditor);
    previousAct->setEnabled(hasEditor);

    windowsMenu->clear();
    createWindowsMenu();
}

void MainWindow::createWindowsMenu()
{
    closeAct->addTo(windowsMenu);
    closeAllAct->addTo(windowsMenu);
    windowsMenu->insertSeparator();
    tileAct->addTo(windowsMenu);
    cascadeAct->addTo(windowsMenu);
    windowsMenu->insertSeparator();
    nextAct->addTo(windowsMenu);
    previousAct->addTo(windowsMenu);

    if (activeEditor()) {
        windowsMenu->insertSeparator();
        windows = workspace->windowList();
        int numVisibleEditors = 0;

        for (int i = 0; i < (int)windows.count(); ++i) {
            QWidget *win = windows.at(i);
            if (!win->isHidden()) {
                QString text = tr("%1 %2")
                               .arg(numVisibleEditors + 1)
                               .arg(win->caption());
                if (numVisibleEditors < 9)
                    text.prepend("&");
                int id = windowsMenu->insertItem(
                              text, this, SLOT(activateWindow(int)));
                bool isActive = (activeEditor() == win);
                windowsMenu->setItemChecked(id, isActive);
                windowsMenu->setItemParameter(id, i);
                ++numVisibleEditors;
            }
        }
    }
}

void MainWindow::activateWindow(int param)
{
    QWidget *win = windows.at(param);
    win->show();
    win->setFocus();
}

void MainWindow::copyAvailable(bool available)
{
    cutAct->setEnabled(available);
    copyAct->setEnabled(available);
}

void MainWindow::updateModIndicator()
{
    if (activeEditor() && activeEditor()->isModified())
        modLabel->setText(tr("MOD"));
    else
        modLabel->clear();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    workspace->closeAllWindows();
    if (activeEditor())
        event->ignore();
    else
        event->accept();
}
